import { createRoot } from "react-dom/client";
import App from "./App";
import "./index.css";
import { QueryClientProvider } from "@tanstack/react-query";
import { queryClient } from "./lib/queryClient";
import { Toaster } from "./components/ui/toaster";

// Completely disable problematic WebSocket connections to prevent app crashes
const OriginalWebSocket = window.WebSocket;

window.WebSocket = function(url: string | URL, protocols?: string | string[]) {
  const urlStr = typeof url === 'string' ? url : url.toString();
  
  // Block WebSocket connections that cause crashes
  if (urlStr.includes('undefined') || urlStr.includes('localhost:undefined')) {
    console.warn('Blocking problematic WebSocket connection:', urlStr);
    
    // Return a functional mock that doesn't cause errors
    const mockWs = {
      readyState: WebSocket.CLOSED,
      url: urlStr,
      protocol: '',
      extensions: '',
      bufferedAmount: 0,
      binaryType: 'blob' as BinaryType,
      onopen: null,
      onclose: null,
      onerror: null,
      onmessage: null,
      CONNECTING: WebSocket.CONNECTING,
      OPEN: WebSocket.OPEN,
      CLOSING: WebSocket.CLOSING,
      CLOSED: WebSocket.CLOSED,
      close: () => {},
      send: () => {},
      addEventListener: () => {},
      removeEventListener: () => {},
      dispatchEvent: () => false,
    };
    
    return mockWs as any;
  }
  
  // Allow normal WebSocket connections but with error handling
  try {
    const ws = new OriginalWebSocket(url, protocols);
    
    // Add basic error handling without complex wrapping
    ws.addEventListener('error', () => {
      console.warn('WebSocket error occurred');
    });
    
    return ws;
  } catch (error) {
    console.warn('WebSocket creation failed:', error);
    return {
      readyState: WebSocket.CLOSED,
      url: urlStr,
      protocol: '',
      extensions: '',
      bufferedAmount: 0,
      binaryType: 'blob' as BinaryType,
      onopen: null,
      onclose: null,
      onerror: null,
      onmessage: null,
      CONNECTING: WebSocket.CONNECTING,
      OPEN: WebSocket.OPEN,
      CLOSING: WebSocket.CLOSING,
      CLOSED: WebSocket.CLOSED,
      close: () => {},
      send: () => {},
      addEventListener: () => {},
      removeEventListener: () => {},
      dispatchEvent: () => false,
    } as any;
  }
} as any;

// Comprehensive unhandled promise rejection handler
window.addEventListener('unhandledrejection', (event) => {
  // Always prevent unhandled rejections from showing in console as errors
  event.preventDefault();
  
  const reason = event.reason;
  const reasonStr = reason?.toString?.() || '';
  const reasonMsg = reason?.message || '';
  const stack = reason?.stack || '';
  
  // Categorize and handle different types of errors
  if (reasonMsg.includes('Invalid url for WebSocket') || 
      reasonStr.includes('Invalid url for WebSocket') ||
      reasonMsg.includes('localhost:undefined') ||
      reasonStr.includes('localhost:undefined')) {
    console.warn('WebSocket URL validation error handled');
    return;
  }
  
  if (reason instanceof SyntaxError && reasonMsg.includes('string did not match the expected pattern')) {
    console.warn('WebSocket parsing error handled');
    return;
  }
  
  if (reasonStr.includes('WebSocket') || reasonMsg.includes('WebSocket') ||
      reasonStr.includes('ws://') || reasonStr.includes('wss://') ||
      stack.includes('WebSocket') || stack.includes('setupWebSocket')) {
    console.warn('WebSocket connection issue handled');
    return;
  }
  
  if (reasonStr.includes('vite') || reasonStr.includes('hmr') || 
      reasonMsg.includes('vite') || reasonMsg.includes('hmr') ||
      stack.includes('vite') || stack.includes('client:') ||
      stack.includes('fallback') || stack.includes('setupWebSocket')) {
    console.warn('Vite HMR error handled');
    return;
  }
  
  if (reasonMsg.includes('Failed to fetch') || reasonMsg.includes('NetworkError') ||
      reasonMsg.includes('Connection refused') || reasonMsg.includes('ECONNREFUSED') ||
      reasonMsg.includes('bad response from the server')) {
    console.warn('Network connection error handled');
    return;
  }
  
  if (reasonMsg.includes('Loading chunk') || reasonMsg.includes('Loading CSS chunk') ||
      reasonStr.includes('ChunkLoadError')) {
    console.warn('Chunk loading error handled');
    return;
  }
  
  if (reason instanceof TypeError && (reasonMsg.includes('URL') || reasonMsg.includes('Invalid URL'))) {
    console.warn('URL construction error handled');
    return;
  }
  
  // Handle authentication errors gracefully
  if (reasonMsg.includes('401') || reasonMsg.includes('Unauthorized')) {
    console.warn('Authentication error handled');
    return;
  }
  
  // For any other unhandled rejection, log it but don't let it crash the app
  console.warn('Unhandled promise rejection handled:', reasonMsg || reasonStr);
});

// Additional WebSocket error handling for Vite HMR
if (import.meta.hot) {
  import.meta.hot.on('vite:error', (error) => {
    console.warn('Vite HMR error handled:', error);
  });
}

// Register service worker for PWA functionality - load asynchronously
if ('serviceWorker' in navigator && 'requestIdleCallback' in window) {
  requestIdleCallback(() => {
    navigator.serviceWorker.register('/sw.js')
      .catch(() => {
        // Silent fail for development
      });
  });
} else if ('serviceWorker' in navigator) {
  setTimeout(() => {
    navigator.serviceWorker.register('/sw.js')
      .catch(() => {
        // Silent fail for development
      });
  }, 2000);
}

import React from "react";

// Proper React error boundary component
class ErrorBoundary extends React.Component<
  { children: React.ReactNode },
  { hasError: boolean; error?: Error }
> {
  constructor(props: { children: React.ReactNode }) {
    super(props);
    this.state = { hasError: false };
  }

  static getDerivedStateFromError(error: Error) {
    return { hasError: true, error };
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo) {
    console.error('React Error Boundary caught an error:', error, errorInfo);
  }

  render() {
    if (this.state.hasError) {
      return (
        <div style={{ 
          padding: '20px', 
          color: 'white', 
          backgroundColor: '#1a1a1a',
          minHeight: '100vh',
          display: 'flex',
          alignItems: 'center',
          justifyContent: 'center',
          flexDirection: 'column'
        }}>
          <h1>Application Error</h1>
          <p>Something went wrong. Please refresh the page to try again.</p>
          <button 
            onClick={() => window.location.reload()}
            style={{ 
              padding: '10px 20px', 
              marginTop: '20px', 
              backgroundColor: '#8b5cf6', 
              color: 'white', 
              border: 'none', 
              borderRadius: '4px',
              cursor: 'pointer'
            }}
          >
            Refresh Page
          </button>
        </div>
      );
    }

    return this.props.children;
  }
}

try {
  const rootElement = document.getElementById("root");
  if (!rootElement) {
    throw new Error('Root element not found');
  }

  createRoot(rootElement).render(
    <ErrorBoundary>
      <QueryClientProvider client={queryClient}>
        <App />
        <Toaster />
      </QueryClientProvider>
    </ErrorBoundary>
  );
} catch (error) {
  console.error('Failed to render app:', error);
  document.body.innerHTML = `
    <div style="padding: 20px; color: white; background: #1a1a1a; min-height: 100vh; display: flex; align-items: center; justify-content: center; flex-direction: column;">
      <h1>Application Startup Error</h1>
      <p>Failed to initialize the application. Please refresh the page.</p>
      <button onclick="window.location.reload()" style="padding: 10px 20px; margin-top: 20px; background: #8b5cf6; color: white; border: none; border-radius: 4px; cursor: pointer;">
        Refresh Page
      </button>
    </div>
  `;
}
